==> astk-serveur-1.3.3.1.tbz: aster/ASTK/ASTK_SERV/lib/mystring.py <==
# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""Utilities to clean, convert, deal with accentuation of strings...
and to clean bad CR.
"""

__revision__ = "$Id: mystring.py,v 1.8 2006/05/23 15:34:26 mcourtoi Exp $"
__cvstag__ = "$Name: v1_3_3 $"

__all__ = ['accents2code', 'accents2sans', 'code2accents', 'code2sans',
           'clean_string', 'clean_multilines_string',
           'indent', 'cleanCR', 'fclean']

import os

# ----- differ messages translation
def _(mesg):
   return mesg

# ------------------------------------------------------------------------------
s0 = ''
s2 = 'aaaeeeeiioouuuc'
# us = s0.decode('iso-8859-1')
us = ('xe0', 'xe2', 'xe4',
      'xe9', 'xe8', 'xea', 'xeb',
      'xee', 'xef',
      'xf4', 'xf6',
      'xf9', 'xfb', 'xfc', 'xe7')

lls = len(s0)
d_accents2code = {}
d_accents2sans = {}
d_code2accents = {}
d_code2sans    = {}
for i in range(lls):
   d_accents2code[s0[i]] = us[i]
   d_accents2sans[s0[i]] = s2[i]
   d_code2accents[us[i]] = s0[i]
   d_code2sans[us[i]]    = s2[i]


# ------------------------------------------------------------------------------
def accents2code(string):
   """Convertir une chaine accentue en une pseudo-chaine unicode
   """
   for code in d_accents2code.keys():
      string = string.replace(code, d_accents2code[code])
   return string

# ------------------------------------------------------------------------------
def code2accents(string):
   """Convertir une pseudo-chaine unicode en une chaine accentue
   """
   for code in d_code2accents.keys():
      string = string.replace(code, d_code2accents[code])
   return string

# ------------------------------------------------------------------------------
def code2sans(string):
   """Convertir une pseudo-chaine unicode en une chaine sans accent
   """
   for code in d_code2sans.keys():
      string = string.replace(code, d_code2sans[code])
   return string

# ------------------------------------------------------------------------------
def accents2sans(string):
   """Convertir une chaine accentue en une chaine sans accent
   """
   for code in d_accents2sans.keys():
      string = string.replace(code, d_accents2sans[code])
   return string

# ------------------------------------------------------------------------------
def utf2latin(string):
   """Dcode une chaine utf8 en latin1
   """
   try:
      ustr = string.decode('utf8')
   except UnicodeDecodeError:
      ustr = decode_chars(string, 'utf8')
   for code in d_accents2sans.keys():
      ustr = ustr.replace(code.decode('latin1'), d_accents2sans[code])
   try:
      string = ustr.encode('latin1')
   except UnicodeEncodeError:
      string = encode_chars(ustr, 'utf8')
   return string

# ------------------------------------------------------------------------------
def char_by_char(method, string, encoding):
   """Encode/Dcode caractre par caractre.
   Ignore les caractres qui ne passent pas.
   """
   l_s = []
   for char in string:
      try:
         l_s.append(getattr(char, method)(encoding))
      except UnicodeEncodeError, UnicodeDecodeError:
         pass
      except:
         print _('Unexpected error in `char_by_char`')
   return ''.join(l_s)

def decode_chars(string, encoding):
   """Dcode caractre par caractre.
   """
   return char_by_char('decode', string, encoding)

def encode_chars(string, encoding):
   """Dcode caractre par caractre.
   """
   return char_by_char('encode', string, encoding)

# ------------------------------------------------------------------------------
def clean_string(string):
   """Nettoie une chaine
   """
   string = string.strip()
   string = accents2sans(string)
   string = code2sans(string)
   try:
      string = utf2latin(string)
   except UnicodeDecodeError, msg:
      print 'STRING : ', string
      raise UnicodeDecodeError, msg
   return string

# ------------------------------------------------------------------------------
def clean_multilines_string(ch, maxlen=90):
   """Filtre les messages :
      - retour chariot pour limiter la taille des lignes,
      - accents.
   """
   txt = []
   for l in ch.split('\n'):
      while len(l) > maxlen:
         ib = l.rfind(' ', 0, maxlen)
         if ib < 0:
            ib = maxlen
         txt.append(clean_string(l[:ib].strip()))
         l = l[ib:].strip()
      txt.append(clean_string(l.strip()))
   return '\n'.join(txt)

# ------------------------------------------------------------------------------
def indent(text, prefix):
   """Add `prefix` before each line of `text`.
   """
   l_s = [prefix+line for line in text.split(os.linesep)]
   return os.linesep.join(l_s)

# -----------------------------------------------------------------------------
def cleanCR(content):
   """Cette fonction nettoie les retours chariot d'une chaine de caractres.
   """
   # nettoyage : `os.linesep` est le bon sparateur sur cette plate-forme
   # pour conserver le dernier CR, on ajoute un blanc qu'on enlvera aprs.
   content += ' '
   l_new = content.splitlines()
   s_new = os.linesep.join(l_new)[:-1]
   return s_new

# -----------------------------------------------------------------------------
def fclean(filename):
   """Cette fonction nettoie les retours chariot d'un fichier texte.
   On crase le contenu.
   Attention si le fichier n'existe pas : IOError !
   """
   # verif
   filename = filename.strip()
   if not os.path.isfile(filename):
      raise IOError, 'File not found : %s' % filename
   elif not os.access(filename, os.W_OK):
      raise IOError, 'No write access to %s' % filename
   # nettoyage
   content = open(filename, 'r').read()
   open(filename, 'w').write(cleanCR(content))

# ------------------------------------------------------------------------------
if __name__ == '__main__':
   mot = "mots cls rservs + mixte c'est-xe0-dire"
   s = accents2code(mot)
   print mot, '-->', s
   
   x = code2accents(s)
   print s, '-->', x

   x = clean_string(s)
   print s, '-->', x

==> astk-serveur-1.5.2.1.tbz: aster/ASTK/ASTK_SERV/lib/mystring.py <==
# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

"""Utilities to clean, convert, deal with accentuation of strings...
and to clean bad CR.
"""

__revision__ = "$Id: mystring.py 3003 2007-06-06 13:27:21Z courtois $"


__all__ = ['convert', 'cut_long_lines',
           'indent', 'cleanCR', 'fclean',
           'accents2code', 'accents2sans', 'code2accents', 'code2sans',
           'clean_string', 'clean_multilines_string']

import os

# ----- differ messages translation
def _(mesg):
   return mesg

# ------------------------------------------------------------------------------
def convert(content, in_encoding, out_encoding):
   """Convertir `content` encod en `in_encoding` vers `out_encoding`.
   """
   if type(content) not in (str, unicode):
      return content
   us = content.decode(in_encoding, 'replace')
   return us.encode(out_encoding, 'replace')

# ------------------------------------------------------------------------------
def to_utf8(content):
   """Retourne `content` (suppos tre en latin-1) en utf-8.
   """
   return convert(content, 'iso-8859-1', 'utf-8')

# ------------------------------------------------------------------------------
def to_latin1(content):
   """Retourne `content` (suppos tre en utf-8) en latin-1.
   """
   return convert(content, 'utf-8', 'iso-8859-1')

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
s0 = ''
s2 = 'aaaeeeeiioouuuc'
# us = s0.decode('iso-8859-1')
us = ('xe0', 'xe2', 'xe4',
      'xe9', 'xe8', 'xea', 'xeb',
      'xee', 'xef',
      'xf4', 'xf6',
      'xf9', 'xfb', 'xfc', 'xe7')

lls = len(s0)
d_accents2code = {}
d_accents2sans = {}
d_code2accents = {}
d_code2sans    = {}
for i in range(lls):
   d_accents2code[s0[i]] = us[i]
   d_accents2sans[s0[i]] = s2[i]
   d_code2accents[us[i]] = s0[i]
   d_code2sans[us[i]]    = s2[i]


# ------------------------------------------------------------------------------
def accents2code(string):
   """Convertir une chaine accentue en une pseudo-chaine unicode
   """
   for code in d_accents2code.keys():
      string = string.replace(code, d_accents2code[code])
   return string

# ------------------------------------------------------------------------------
def code2accents(string):
   """Convertir une pseudo-chaine unicode en une chaine accentue
   """
   for code in d_code2accents.keys():
      string = string.replace(code, d_code2accents[code])
   return string

# ------------------------------------------------------------------------------
def code2sans(string):
   """Convertir une pseudo-chaine unicode en une chaine sans accent
   """
   for code in d_code2sans.keys():
      string = string.replace(code, d_code2sans[code])
   return string

# ------------------------------------------------------------------------------
def accents2sans(string):
   """Convertir une chaine accentue en une chaine sans accent
   """
   for code in d_accents2sans.keys():
      string = string.replace(code, d_accents2sans[code])
   return string

# ------------------------------------------------------------------------------
def utf2latin(string):
   """Dcode une chaine utf8 en latin1
   """
   try:
      ustr = string.decode('utf8')
   except UnicodeDecodeError:
      ustr = decode_chars(string, 'utf8')
   for code in d_accents2sans.keys():
      ustr = ustr.replace(code.decode('latin1'), d_accents2sans[code])
   try:
      string = ustr.encode('latin1')
   except UnicodeEncodeError:
      string = encode_chars(ustr, 'utf8')
   return string

# ------------------------------------------------------------------------------
def char_by_char(method, string, encoding):
   """Encode/Dcode caractre par caractre.
   Ignore les caractres qui ne passent pas.
   """
   l_s = []
   for char in string:
      try:
         l_s.append(getattr(char, method)(encoding))
      except UnicodeEncodeError, UnicodeDecodeError:
         pass
      except:
         print _('Unexpected error in `char_by_char`')
   return ''.join(l_s)

def decode_chars(string, encoding):
   """Dcode caractre par caractre.
   """
   return char_by_char('decode', string, encoding)

def encode_chars(string, encoding):
   """Dcode caractre par caractre.
   """
   return char_by_char('encode', string, encoding)

# ------------------------------------------------------------------------------
def clean_string(string):
   """Nettoie une chaine
   """
   string = string.strip()
   string = accents2sans(string)
   string = code2sans(string)
   try:
      string = utf2latin(string)
   except UnicodeDecodeError, msg:
      print 'STRING : ', string
      raise UnicodeDecodeError, msg
   return string

# ------------------------------------------------------------------------------
def clean_multilines_string(ch, maxlen=90):
   """Filtre les messages :
      - retour chariot pour limiter la taille des lignes,
      - accents.
   """
   txt = []
   for l in ch.split('\n'):
      while len(l) > maxlen:
         ib = l.rfind(' ', 0, maxlen)
         if ib < 0:
            ib = maxlen
         txt.append(clean_string(l[:ib].strip()))
         l = l[ib:].strip()
      txt.append(clean_string(l.strip()))
   return '\n'.join(txt)

# ------------------------------------------------------------------------------
def indent(text, prefix):
   """Add `prefix` before each line of `text`.
   """
   l_s = [prefix+line for line in text.split(os.linesep)]
   return os.linesep.join(l_s)

# -----------------------------------------------------------------------------
def cleanCR(content):
   """Cette fonction nettoie les retours chariot d'une chaine de caractres.
   """
   # nettoyage : `os.linesep` est le bon sparateur sur cette plate-forme
   # pour conserver le dernier CR, on ajoute un blanc qu'on enlvera aprs.
   content += ' '
   l_new = content.splitlines()
   s_new = os.linesep.join(l_new)[:-1]
   return s_new

# -----------------------------------------------------------------------------
def fclean(filename):
   """Cette fonction nettoie les retours chariot d'un fichier texte.
   On crase le contenu.
   Attention si le fichier n'existe pas : IOError !
   """
   # verif
   filename = filename.strip()
   if not os.path.isfile(filename):
      raise IOError, 'File not found : %s' % filename
   elif not os.access(filename, os.W_OK):
      raise IOError, 'No write access to %s' % filename
   # nettoyage
   content = open(filename, 'r').read()
   open(filename, 'w').write(cleanCR(content))

# -----------------------------------------------------------------------------
def maximize_lines(l_fields, maxlen, sep):
   """Construit des lignes dont la longueur est au plus de `maxlen` caractres.
   Les champs sont assembls avec le sparateur `sep`.
   """
   newlines = []
   # ceinture
   assert max([len(f) for f in l_fields]) <= maxlen, 'lignes trop longues : %s' % l_fields
   while len(l_fields) > 0:
      cur = []
      while len(l_fields) > 0 and len(sep.join(cur + [l_fields[0],])) <= maxlen:
         cur.append(l_fields.pop(0))
      # bretelle
      assert len(cur) > 0, l_fields
      newlines.append(sep.join(cur))
   newlines = [l for l in newlines if l != '']
   return newlines

# -----------------------------------------------------------------------------
def force_split(txt, maxlen):
   """Force le dcoupage de la ligne  'maxlen' caractres.
   """
   l_res = []
   while len(txt) > maxlen:
      l_res.append(txt[:maxlen])
      txt = txt[maxlen:]
   l_res.append(txt)
   return l_res

# -----------------------------------------------------------------------------
def cut_long_lines(txt, maxlen, sep=os.linesep,
                   l_separ=(' ', ',', ';', '.', ':')):
   """Coupe les morceaux de `txt` (isols avec `sep`) de plus de `maxlen`
   caractres.
   On utilise successivement les sparateurs de `l_separ`.
   """
   l_lines = txt.split(sep)
   newlines = []
   for line in l_lines:
      if len(line) > maxlen:
         l_sep = list(l_separ)
         if len(l_sep) == 0:
            newlines.extend(force_split(line, maxlen))
            continue
         else:
            line = cut_long_lines(line, maxlen, l_sep[0], l_sep[1:])
            line = maximize_lines(line, maxlen, l_sep[0])
         newlines.extend(line)
      else:
         newlines.append(line)
   # au plus haut niveau, on assemble le texte
   if sep == os.linesep:
      newlines = os.linesep.join(newlines)
   return newlines

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
if __name__ == '__main__':
   su8 = 'texte accentué en français'
   print su8
   sl1 = to_latin1(su8)
   print sl1
   bis = to_latin1(sl1)
   print bis
   
   dl1 = 'dj accentu'
   print dl1
   du8 = to_utf8(dl1)
   print du8
   bis = to_utf8(du8)
   print du8
